import unittest

from cubicweb.devtools.testlib import CubicWebTC
from cubicweb.devtools.htmlparser import XMLValidator


class BoostrapLogFormTemplateTC(CubicWebTC):
    def _login_labels(self):
        valid = self.content_type_validators.get("text/html", XMLValidator)()
        with self.new_access("anon").web_request() as req:
            page = valid.parse_string(self.vreg["views"].main_template(req, "login"))
        return page.find_tag("label")

    def test_label(self):
        self.set_option("allow-email-login", "yes")
        self.assertEqual(self._login_labels(), ["login or email", "password"])
        self.set_option("allow-email-login", "no")
        self.assertEqual(self._login_labels(), ["login", "password"])


if __name__ == "__main__":
    unittest.main()



"""fluid_design_system implementation of reledit

:organization: Logilab
:copyright: 2013-2014 LOGILAB S.A. (Paris, FRANCE), license is LGPL.
:contact: https://www.logilab.fr/ -- mailto:contact@logilab.fr
"""
__docformat__ = "restructuredtext en"

from cubicweb.web.views.reledit import AutoClickAndEditFormView

AutoClickAndEditFormView._addzone = ' <span class="glyphicon glyphicon-plus"></span>'
AutoClickAndEditFormView._deletezone = (
    ' <span class="glyphicon glyphicon-remove"></span>'
)
AutoClickAndEditFormView._editzone = ' <span class="glyphicon glyphicon-pencil"></span>'

try:
    from cubicweb_inlinedit.views import reledit
except ImportError:
    pass
else:
    reledit.ReleditRelationFormHandler._addzone = (
        ' <span class="glyphicon glyphicon-plus"></span>'
    )
    reledit.ReleditFormHandler._editzone = (
        ' <span class="glyphicon glyphicon-pencil"></span>'
    )
    reledit.ReleditEntityFormHandler._deletezone = (
        ' <span class="glyphicon glyphicon-remove"></span>'
    )



"""fluid_design_system implementation of tabs

:organization: Logilab
:copyright: 2014 LOGILAB S.A. (Paris, FRANCE), license is LGPL.
:contact: https://www.logilab.fr/ -- mailto:contact@logilab.fr
"""

__docformat__ = "restructuredtext en"

from logilab.common.decorators import monkeypatch

from cubicweb import utils
from cubicweb.web.views import tabs


@monkeypatch(tabs.TabsMixin)
def render_tabs(self, tabs, default, entity=None):
    # delegate to the default tab if there is more than one entity
    # in the result set (tabs are pretty useless there)
    if entity and len(self.cw_rset) > 1:
        entity.view(default, w=self.w)
        return
    self._cw.add_js(("cubicweb.ajax.js", "jquery.cookie.js"))
    # prune tabs : not all are to be shown
    tabs, active_tab = self.prune_tabs(tabs, default)
    # build the html structure
    uid = entity and entity.eid or utils.make_uid("tab")
    self.w('<div id="entity-tabs-%s" class="entity-tabs">', uid)
    self.w('<ul class="nav nav-tabs">')
    for i, (tabid, domid, tabkwargs) in enumerate(tabs):
        if domid == active_tab:
            self.w('<li class="active">')
        else:
            self.w("<li>")
        self.w('<a href="#%s" data-toggle="tab">', domid)
        self.w(tabkwargs.pop("label", self._cw._(tabid)))
        self.w("</a>")
        self.w("</li>")
    self.w("</ul>")
    self.w('<div class="tab-content">')
    for tabid, domid, tabkwargs in tabs:
        active = "active" if domid == active_tab else ""
        self.w('<div id="%s" class="tab-pane %s">', domid, active)
        if self.lazy:
            tabkwargs.setdefault("tabid", domid)
            tabkwargs.setdefault("vid", tabid)
            self.lazyview(**tabkwargs)
        else:
            self._cw.view(tabid, w=self.w, **tabkwargs)
        self.w("</div>")
    self.w("</div>")
    self.w("</div>")
    # XXX make work history: true
    if self.lazy:
        self._cw.add_onload(
            """
        $('#entity-tabs-%(uid)s a[data-toggle="tab"]').on('shown.bs.tab', function (e) {
            var tabid = $(e.target).attr('href').slice(1);
            setTab(tabid, '%(cookiename)s');
        });
        setTab('%(domid)s', '%(cookiename)s');
        """
            % {"domid": active_tab, "uid": uid, "cookiename": self.cookie_name}
        )



# copyright 2013-2021 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact https://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <https://www.gnu.org/licenses/>.

"""cubicweb-fluid-design-system automatic tests


uncomment code below if you want to activate automatic test for your cube:

.. sourcecode:: python

    from cubicweb.devtools.testlib import AutomaticWebTest

    class AutomaticWebTest(AutomaticWebTest):
        '''provides `to_test_etypes` and/or `list_startup_views` implementation
        to limit test scope
        '''

        def to_test_etypes(self):
            '''only test views for entities of the returned types'''
            return set(('My', 'Cube', 'Entity', 'Types'))

        def list_startup_views(self):
            '''only test startup views of the returned identifiers'''
            return ('some', 'startup', 'views')
"""

from logilab.common.testlib import TestCase

# this will make sure cubes are in our sys.path
import cubicweb.devtools  # noqa

from cubicweb_fluid_design_system import monkeypatch_default_value


class MonkeypatchDefaultValueTC(TestCase):
    def test_function(self):
        def func(a, b, c=0, d=None, e=False, f=()):
            return c

        self.assertEqual(func(1, 2), 0)
        monkeypatch_default_value(func, "c", 42)
        self.assertEqual(func(1, 2), 42)

    def test_method(self):
        class Class(object):
            def meth(self, a, b, c=0, d=None, e=False, f=()):
                return c

        obj = Class()
        self.assertEqual(obj.meth(1, 2), 0)
        monkeypatch_default_value(Class.meth, "c", 42)
        self.assertEqual(obj.meth(1, 2), 42)


if __name__ == "__main__":
    import unittest

    unittest.main()



# -*- coding: utf-8 -*-
"""fluid_design_system implementation of base templates

:organization: Logilab
:copyright: 2013 LOGILAB S.A. (Paris, FRANCE), license is LGPL.
:contact: https://www.logilab.fr/ -- mailto:contact@logilab.fr
"""

__docformat__ = "restructuredtext en"

from logilab.common.decorators import monkeypatch
from logilab.mtconverter import xml_escape

from cubicweb import _
from cubicweb.web import formwidgets as fw
from cubicweb.web.views import basetemplates, basecomponents, actions

from cubicweb.schema import display_name
from cubicweb.utils import UStringIO

from cubicweb.web.views.boxes import SearchBox


basetemplates.LogForm.needs_css = ()
basetemplates.LogForm.form_buttons = [
    fw.ResetButton(
        label=_("cancel"), attrs={"class": "btn btn-default", "data-dismiss": "modal"}
    ),
    fw.SubmitButton(label=_("log in"), attrs={"class": "btn btn-primary"}),
]

basetemplates.LogForm.form_renderer_id = "modal-form-renderer"


@monkeypatch(basetemplates.LogFormView)
def call(self, id, klass, title=True, showmessage=True, showonload=True):
    stitle = "&#160;"
    if title:
        stitle = self._cw.property_value("ui.site-title")
        if stitle:
            stitle = xml_escape(stitle)
    config = self._cw.vreg.config
    if config["auth-mode"] != "http":
        self.login_form(
            id, "loginModal", stitle, showmessage, showonload
        )  # Cookie authentication
    if config.anonymous_user()[0] and config.get("https-deny-anonymous"):
        path = xml_escape(config["base-url"] + self._cw.relative_path())
        self.w(
            '<div class="loginMessage alert"><a href="%s">%s</a></div>\n',
            path,
            self._cw._("No account? Try public access at %s") % path,
        )


@monkeypatch(basetemplates.LogFormView)
def login_form(self, id, modal_id, title, showmessage, showonload):
    cw = self._cw
    form = cw.vreg["forms"].select("logform", cw)
    if showonload:
        # remove cancel button on ?vid=login
        form.form_buttons = form.form_buttons[1:]
    if cw.vreg.config["allow-email-login"]:
        label = cw._("login or email")
    else:
        label = cw.pgettext("CWUser", "login")
    form.field_by_name("__login").label = label
    form.render(
        w=self.w,
        table_class="",
        display_progress_div=False,
        modal_id=modal_id,
        title=title,
        showmessage=showmessage,
        showonload=showonload,
    )
    cw.html_headers.add_onload('jQuery("#__login:visible").focus()')


HTML5 = "<!DOCTYPE html>"

basetemplates.TheMainTemplate.doctype = HTML5

# options which can be changed freely
basetemplates.TheMainTemplate.twbs_container_cls = "container-fluid"
basetemplates.TheMainTemplate.twbs_col_cls = "col-xs-"
basetemplates.TheMainTemplate.twbs_col_size = 2

# options which require recompiling bootstrap.css from source
basetemplates.TheMainTemplate.twbs_grid_columns = 12


@monkeypatch(basetemplates.TheMainTemplate)  # noqa: F811
def call(self, view):
    self.set_request_content_type()
    self.template_header(self.content_type, view)
    self.template_page_content(view)


@monkeypatch(basetemplates.TheMainTemplate)
def template_header(
    self, content_type, view=None, page_title="", additional_headers=()
):
    page_title = page_title or view.page_title()
    additional_headers = additional_headers or view.html_headers()
    self.template_html_header(content_type, page_title, additional_headers)


@monkeypatch(basetemplates.TheMainTemplate)
def template_html_header(self, content_type, page_title, additional_headers=()):
    self.write_doctype()
    self._cw.html_headers.define_var("BASE_URL", self._cw.base_url())
    self._cw.html_headers.define_var("DATA_URL", self._cw.datadir_url)
    self.whead(
        '<meta http-equiv="content-type" content="%s; charset=%s"/>\n',
        content_type,
        self._cw.encoding,
    )
    self.whead(
        '<meta name="viewport" content="initial-scale=1.0, '
        'maximum-scale=1.0, width=device-width"/>'
    )
    self.whead("\n".join(additional_headers) + "\n")
    self.wview("htmlheader", rset=self.cw_rset)
    if page_title:
        self.whead("<title>%s</title>\n", page_title)


@monkeypatch(basetemplates.TheMainTemplate)
def template_page_content(self, view):
    self.w("<body>\n")
    self.wview("header", rset=self.cw_rset, view=view)
    self.w('<div id="page" class="%s">\n', self.twbs_container_cls)
    self.w('<div class="row">\n')
    left_boxes = list(
        self._cw.vreg["ctxcomponents"].poss_visible_objects(
            self._cw, rset=self.cw_rset, view=view, context="left"
        )
    )
    right_boxes = list(
        self._cw.vreg["ctxcomponents"].poss_visible_objects(
            self._cw, rset=self.cw_rset, view=view, context="right"
        )
    )
    nb_boxes = int(bool(left_boxes)) + int(bool(right_boxes))
    content_cols = self.twbs_grid_columns
    if nb_boxes:
        content_cols = self.twbs_grid_columns - self.twbs_col_size * nb_boxes
    self.nav_column(view, left_boxes, "left")
    self.content_column(view, content_cols)
    self.nav_column(view, right_boxes, "right")
    self.w("</div>\n")  # closes class=row
    self.w("</div>\n")  # closes id="page" from template_page_content
    self.template_footer(view)
    self.w("</body>\n")


@monkeypatch(basetemplates.TheMainTemplate)  # noqa: F811
def get_components(self, view, context):
    ctxcomponents = self._cw.vreg["ctxcomponents"]
    return ctxcomponents.poss_visible_objects(
        self._cw, rset=self.cw_rset, view=view, context=context
    )


@monkeypatch(basetemplates.TheMainTemplate)
def state_header(self):
    state = self._cw.search_state
    if state[0] == "normal":
        return
    _ = self._cw._
    value = self._cw.view("oneline", self._cw.eid_rset(state[1][1]))
    target, eid, r_type, searched_type = self._cw.search_state[1]
    cancel_link = """<a href="%(url)s" role="button"
    class="btn btn-default" title="%(title)s">%(title)s</a>""" % {
        "url": self._cw.build_url(str(eid), vid="edition", __mode="normal"),
        "title": _("cancel"),
    }
    msg = " ".join(
        (
            _("searching for"),
            '<strong>"%s"</strong>' % display_name(self._cw, state[1][3]),
            _("to associate with"),
            value,
            _("by relation"),
            '<strong>"%s"</strong>' % display_name(self._cw, state[1][2], state[1][0]),
            cancel_link,
        )
    )
    return self.w('<div class="alert alert-info">%s</div>', msg, escape=False)


@monkeypatch(basetemplates.TheMainTemplate)
def nav_column(self, view, boxes, context):
    if boxes:
        stream = UStringIO()
        for box in boxes:
            box.render(w=stream.write, view=view)
        html = stream.getvalue()
        if html:
            # only display aside columns if html availble
            self.w(
                '<aside id="aside-main-%s" class="%s%s cwjs-aside">\n',
                context,
                self.twbs_col_cls,
                self.twbs_col_size,
            )
            self.w(html)
            self.w("</aside>\n")
    return len(boxes)


@monkeypatch(basetemplates.TheMainTemplate)
def content_column(self, view, content_cols):
    self.w(
        '<div id="main-center" class="%s%s" role="main">',
        self.twbs_col_cls,
        content_cols,
    )
    components = self._cw.vreg["components"]
    self.content_components(view, components)
    self.w('<div id="pageContent">')
    self.content_header(view)
    vtitle = self._cw.form.get("vtitle")
    if vtitle:
        self.w('<div class="vtitle">%s</div>\n', vtitle)
    self.state_header()
    self.content_navrestriction_components(view, components)
    nav_html = UStringIO()
    if view and not view.handle_pagination:
        view.paginate(w=nav_html.write)
    self.w(nav_html.getvalue())
    self.w('<div id="contentmain">\n')
    view.render(w=self.w)
    self.w("</div>\n")  # closes id=contentmain
    self.w(nav_html.getvalue())
    self.content_footer(view)
    self.w("</div>\n")  # closes div#pageContent
    self.w("</div>\n")  # closes div.%(prefix)s-%(col)s


@monkeypatch(basetemplates.TheMainTemplate)
def content_components(self, view, components):
    """TODO : should use context"""
    rqlcomp = components.select_or_none("rqlinput", self._cw, rset=self.cw_rset)
    if rqlcomp:
        rqlcomp.render(w=self.w, view=view)
    msgcomp = components.select_or_none("applmessages", self._cw, rset=self.cw_rset)
    if msgcomp:
        msgcomp.render(w=self.w)


@monkeypatch(basetemplates.TheMainTemplate)
def content_navrestriction_components(self, view, components):
    # display entity type restriction component
    etypefilter = components.select_or_none(
        "etypenavigation", self._cw, rset=self.cw_rset
    )
    if etypefilter and etypefilter.cw_propval("visible"):
        etypefilter.render(w=self.w)


@monkeypatch(basetemplates.TheMainTemplate)
def template_footer(self, view=None):
    self.wview("footer", rset=self.cw_rset, view=view)


# main header

basecomponents.ApplLogo.context = "header-logo"
# use basecomponents.ApplicationName.visible = False
basecomponents.ApplicationName.context = "header-left"
basecomponents.ApplLogo.order = 1
basecomponents.ApplicationName.order = 10
basecomponents.CookieLoginComponent.order = 10
basecomponents.AuthenticatedUserStatus.order = 5
SearchBox.order = -1
SearchBox.context = "header-right"
SearchBox.layout_id = "simple-layout"


@monkeypatch(basetemplates.HTMLPageHeader)  # noqa: F811
def call(self, view, **kwargs):
    self.main_header(view)
    self.breadcrumbs(view)


def get_components(self, view, context):  # noqa: F811
    ctxcomponents = self._cw.vreg["ctxcomponents"]
    return ctxcomponents.poss_visible_objects(
        self._cw, rset=self.cw_rset, view=view, context=context
    )


basetemplates.HTMLPageHeader.get_components = get_components
basetemplates.HTMLPageHeader.css = {
    "navbar-extra": "navbar-default",
    "breadcrumbs": "cw-breadcrumb",
    "container-cls": basetemplates.TheMainTemplate.twbs_container_cls,
    "header-left": "",
    "header-right": "d-flex justify-content-end align-items-center m-2 navbar-right list-unstyled",
}


@monkeypatch(basetemplates.HTMLPageHeader)
def main_header(self, view):
    self.w('<nav class="navbar %s" role="banner">', self.css["navbar-extra"])
    self.w('<div class="%s">', self.css["container-cls"])
    self.display_navbar_header(self.w, view)
    self.w('<div id="tools-group" class="collapse navbar-collapse">')
    self.display_header_components(self.w, view, "header-left")
    self.display_header_components(self.w, view, "header-right")
    self.w("</div></div></nav>")


def display_navbar_header(self, w, view):
    w(
        """<div class="navbar-header">
    <button class="navbar-toggle" data-target="#tools-group" data-toggle="collapse" type="button">
    <span class="sr-only">%s</span>
    <span class="icon-bar"></span>
    <span class="icon-bar"></span>
    <span class="icon-bar"></span>
    </button>"""
        % self._cw._("Toggle navigation")
    )
    components = self.get_components(view, context="header-logo")
    if components:
        for component in components:
            component.render(w=w)
    w("</div>")


basetemplates.HTMLPageHeader.display_navbar_header = display_navbar_header


def display_header_components(self, w, view, context):
    components = self.get_components(view, context=context)
    if components:
        w('<ul class="%s">' % self.css[context])
        for component in components:
            w("<li>")
            component.render(w=w)
            w("</li>")
        w("</ul>")


basetemplates.HTMLPageHeader.display_header_components = display_header_components


@monkeypatch(basetemplates.HTMLPageHeader)
def breadcrumbs(self, view):
    components = self.get_components(view, context="header-center")
    if components:
        self.w(
            '<nav role="navigation" class="%s">',
            self.css.get("breadcrumbs", "breadcrumbs-defaul"),
        )
        for component in components:
            component.render(w=self.w)
        self.w("</nav>")


@monkeypatch(basetemplates.HTMLContentFooter)  # noqa: F811
def call(self, view, **kwargs):
    components = self._cw.vreg["ctxcomponents"].poss_visible_objects(
        self._cw, rset=self.cw_rset, view=view, context="navbottom"
    )
    if components:
        # the row is needed here to correctly put the HTML flux
        self.w('<div id="contentfooter">')
        for comp in components:
            comp.render(w=self.w, view=view)
        self.w("</div>")


@monkeypatch(basetemplates.HTMLPageFooter)  # noqa: F811
def call(self, **kwargs):
    self.w('<footer id="pagefooter" role="contentinfo">')
    self.footer_content()
    self.w("</footer>\n")


def registration_callback(vreg):
    vreg.register_all(globals().values(), __name__)
    vreg.unregister(actions.CancelSelectAction)



# pylint: disable=W0622
"""cubicweb-fluid-design-system application packaging information"""

modname = "fluid_design_system"
distname = "cubicweb-fluid-design-system"

numversion = (1, 9, 1)
version = ".".join(str(num) for num in numversion)

license = "LGPL"
author = "LOGILAB S.A. (Paris, FRANCE)"
author_email = "contact@logilab.fr"
description = ""
web = "https://www.cubicweb.org/project/%s" % distname

__depends__ = {
    "cubicweb": ">= 3.35.0",
}
__recommends__ = {}

classifiers = [
    "Environment :: Web Environment",
    "Framework :: CubicWeb",
    "Programming Language :: Python",
    "Programming Language :: Python :: 3",
    "Programming Language :: JavaScript",
]



"""fluid_design_system implementation of formfields

:organization: Logilab
:copyright: 2015 LOGILAB S.A. (Paris, FRANCE), license is LGPL.
:contact: https://www.logilab.fr/ -- mailto:contact@logilab.fr
"""

__docformat__ = "restructuredtext en"

from logilab.common.decorators import monkeypatch

from cubicweb.utils import UStringIO
from cubicweb.web import formfields


@monkeypatch(formfields.FileField)
def render_subfield(self, form, field, renderer):
    data = UStringIO()  # XXX add support for tracewrites
    w = data.write
    w('<div class="row">')
    w(renderer.render_label(form, field))
    w('<div class="col-md-9">')
    w(field.render(form, renderer))
    w("</div>")
    w("</div>")
    w('<div class="row">')
    w('<div class="col-md-offset-3 col-md-9">')
    w(renderer.render_help(form, field))
    w("</div>")
    w("</div>")
    return data.getvalue()



"""fluid_design_system implementation of formwidgets

:organization: Logilab
:copyright: 2013 LOGILAB S.A. (Paris, FRANCE), license is LGPL.
:contact: https://www.logilab.fr/ -- mailto:contact@logilab.fr
"""

__docformat__ = "restructuredtext en"

from logilab.common.decorators import monkeypatch
from logilab.mtconverter import xml_escape

from cubicweb import tags, __version__ as cwver
from cubicweb.web import formwidgets


# Buttons
@monkeypatch(formwidgets.Button)
def render(self, form, field=None, renderer=None):
    label = form._cw._(self.label)
    attrs = self.attrs.copy()
    if not cwver.startswith("3.20"):
        # 3.21 commit b0417cacecd9
        attrs.setdefault("class", self.css_class)
    if self.cwaction:
        assert self.onclick is None
        attrs["onclick"] = "postForm('__action_%s', '%s', '%s')" % (
            self.cwaction,
            self.label,
            form.domid,
        )
    elif self.onclick:
        attrs["onclick"] = self.onclick
    if self.name:
        attrs["name"] = self.name
        if self.setdomid:
            attrs["id"] = self.name
    if self.icon:
        img = '<i class="%s"> </i>' % self.icon
    else:
        img = ""
    return tags.button(
        img + xml_escape(label),
        escapecontent=False,
        value=label,
        type=self.type,
        **attrs,
    )



# -*- coding: utf-8 -*-
"""fluid_design_system implementation of tableview

:organization: Logilab
:copyright: 2013 LOGILAB S.A. (Paris, FRANCE), license is LGPL.
:contact: https://www.logilab.fr/ -- mailto:contact@logilab.fr
"""

__docformat__ = "restructuredtext en"

from logilab.common.decorators import monkeypatch

from cubicweb.web.views import tableview

tableview.TableLayout.cssclass = "listing table table-bordered table-condensed"


_row_attributes = tableview.TableLayout.row_attributes


@monkeypatch(tableview.TableLayout)
def row_attributes(self, rownum):
    attrs = _row_attributes(self, rownum)
    del attrs["onmouseover"]
    del attrs["onmouseout"]
    return attrs



"""fluid_design_system implementation of autoforms

:organization: Logilab
:copyright: 2013 LOGILAB S.A. (Paris, FRANCE), license is LGPL.
:contact: https://www.logilab.fr/ -- mailto:contact@logilab.fr
"""

__docformat__ = "restructuredtext en"

from logilab.common.decorators import monkeypatch
from cubicweb.web.views import editforms


@monkeypatch(editforms.EditionFormView)
def form_title(self, entity):
    """the form view title"""
    ptitle = self._cw._(self.title)
    self.w(
        '<div class="formTitle"><h1>%s %s</h1></div>',
        entity.dc_type(),
        ptitle and "(%s)" % ptitle,
        escape=False,
    )


@monkeypatch(editforms.CreationFormView)  # noqa: F811
def form_title(self, entity):
    """the form view title"""
    if "__linkto" in self._cw.form:
        if isinstance(self._cw.form["__linkto"], list):
            # XXX which one should be considered (case: add a ticket to a
            # version in jpl)
            rtype, linkto_eid, role = self._cw.form["__linkto"][0].split(":")
        else:
            rtype, linkto_eid, role = self._cw.form["__linkto"].split(":")
        linkto_rset = self._cw.eid_rset(linkto_eid)
        linkto_type = linkto_rset.description[0][0]
        if role == "subject":
            title = self._cw.__(
                "creating %s (%s %s %s %%(linkto)s)"
                % (entity.e_schema, entity.e_schema, rtype, linkto_type)
            )
        else:
            title = self._cw.__(
                "creating %s (%s %%(linkto)s %s %s)"
                % (entity.e_schema, linkto_type, rtype, entity.e_schema)
            )
        msg = title % {"linkto": self._cw.view("incontext", linkto_rset)}
        # bs customization begins
        self.w(
            '<div class="formTitle notransform"><h1>%s</h1></div>', msg, escape=False
        )
        # bs customization ends
    else:
        super(editforms.CreationFormView, self).form_title(entity)



"""fluid_design_system implementation of tag cube

:organization: Logilab
:copyright: 2015 LOGILAB S.A. (Paris, FRANCE), license is LGPL.
:contact: https://www.logilab.fr/ -- mailto:contact@logilab.fr
"""

try:
    from cubicweb_tag.views import SimilarityBox
except ImportError:
    pass
else:
    from logilab.common.decorators import monkeypatch
    from cubicweb import tags

    @monkeypatch(SimilarityBox)
    def render_body(self, w):
        # bs customization begins
        self._cw.view("list", self.cw_rset, w=w, klass="list-unstyled")
        # bs customization ends
        rql = self.rql % "" % {"x": self.entity.eid}
        title = self._cw._("entities similar to %s") % self.entity.dc_title()
        url = self._cw.build_url("view", rql=rql, vtitle=title)
        w("<div>[%s]</div>" % tags.a(self._cw._("see all"), href=url))



# -*- coding: utf-8 -*-
"""fluid_design_system implementation of startup views

:organization: Logilab
:copyright: 2013 LOGILAB S.A. (Paris, FRANCE), license is LGPL.
:contact: https://www.logilab.fr/ -- mailto:contact@logilab.fr
"""

__docformat__ = "restructuredtext en"

from logilab.common.decorators import monkeypatch
from logilab.common.textutils import unormalize
from logilab.mtconverter import xml_escape

from cubicweb.web.views import startup, uicfg

startup.ManageView.box_html = """<div class="panel panel-default">
    <div class="panel-heading">
        <div class="panel-title">%s</div>
    </div>
    <div class="panel-body">%s</div>
    </div>"""


@monkeypatch(startup.ManageView)
def manage_actions(self):
    allactions = self._cw.vreg["actions"].possible_actions(self._cw)
    if allactions.get("manage"):
        html = []
        html.append("<ul>")
        for action in allactions["manage"]:
            html.append(
                '<li><a href="%s">%s</a></li>'
                % (action.url(), self._cw._(action.title))
            )
        html.append("</ul>")
        self.w(
            self.box_html,
            self._cw._("Manage"),
            "\n".join(html),
            escape=False,
        )


@monkeypatch(startup.ManageView)
def startup_views(self):
    views = [
        v
        for v in self._cw.vreg["views"].possible_views(self._cw, None)
        if v.category == "startupview" and v.__regid__ not in self.skip_startup_views
    ]
    if not views:
        return
    html = []
    html.append("<ul>")
    for v in sorted(views, key=lambda x: self._cw._(x.title)):
        html.append(
            '<li><a href="%s">%s</a></li>'
            % (xml_escape(v.url()), xml_escape(self._cw._(v.title).capitalize()))
        )
    html.append("</ul>")
    self.w(
        self.box_html,
        self._cw._("Startup views"),
        "\n".join(html),
        escape=False,
    )


@monkeypatch(startup.ManageView)
def entities(self):
    schema = self._cw.vreg.schema
    eschemas = [
        eschema
        for eschema in schema.entities()
        if uicfg.indexview_etype_section.get(eschema) == "application"
    ]
    if eschemas:
        html = self.entity_types_table(eschemas)
        self.w(
            self.box_html,
            self._cw._("Browse by entity type"),
            "\n".join(html),
            escape=False,
        )


@monkeypatch(startup.ManageView)
def entity_types_table(self, eschemas):
    infos = sorted(self.entity_types(eschemas), key=lambda t: unormalize(t[0]))
    q, r = divmod(len(infos), 2)
    html = []
    html.append('<div class="row">')
    for links in (infos[: q + r], infos[q + r :]):
        if links:
            html.append('<div class="col-md-6">')
            html.append('<ul class="list-unstyled">')
            for (_, etypelink, addlink) in links:
                html.append("<li>%s %s</li>" % (addlink, etypelink))
            html.append("</ul>")
            html.append("</div>")
    html.append("</div>")
    return html



"""fluid_design_system implementation of cwproperty view

:organization: Logilab
:copyright: 2013 LOGILAB S.A. (Paris, FRANCE), license is LGPL.
:contact: https://www.logilab.fr/ -- mailto:contact@logilab.fr
"""

__docformat__ = "restructuredtext en"

from logilab.common.decorators import monkeypatch
from cubicweb.web.views.cwproperties import SystemCWPropertiesForm, make_togglable_link


@monkeypatch(SystemCWPropertiesForm)
def wrap_main_form(self, group, label, form):
    label += ' <span class="caret"></span>'
    status = self._group_status(group)
    cssclass = "panel-body %s" % status if status else "panel-body"
    self.w(
        '<div class="panel panel-default">' '<div class="panel-heading">%s</div>\n',
        make_togglable_link("fieldset_" + group, label),
        escape=False,
    )
    self.w('<div class="%s" id="fieldset_%s">', cssclass, group)
    self.w(form)
    self.w("</div>")
    self.w("</div>")


@monkeypatch(SystemCWPropertiesForm)
def wrap_grouped_form(self, group, label, objects):
    label += ' <span class="caret"></span>'
    status = self._group_status(group)
    cssclass = "panel-body %s" % status if status else "panel-body"
    self.w(
        '<div class="panel panel-default">' '<div class="panel-heading">%s</div>\n',
        make_togglable_link("fieldset_" + group, label),
        escape=False,
    )
    self.w('<div class="%s" id="fieldset_%s">', cssclass, group)

    sorted_objects = sorted(
        (self._cw.__("%s_%s" % (group, o)), o, f) for o, f in objects.items()
    )
    for label, oid, form in sorted_objects:
        self.wrap_object_form(group, oid, label, form)
    self.w("</div>")
    self.w("</div>")



"""cubicweb-fluid-design-system application package


"""
import inspect


def monkeypatch_default_value(func, arg, value):
    # work on the underlying function object if func is a method, that's
    # where '__defaults__' is actually stored.
    if inspect.ismethod(func):
        func = func.__func__
    getfullargspec = inspect.getfullargspec
    argspec = getfullargspec(func)
    # ArgSpec.args contains regular and named parameters, only keep the latter
    named_args = argspec.args[-len(argspec.defaults) :]
    idx = named_args.index(arg)
    # generate and inject a new '__defaults__' tuple with the new default value
    new_defaults = func.__defaults__[:idx] + (value,) + func.__defaults__[idx + 1 :]
    func.__defaults__ = new_defaults



"""fluid_design_system implementation of primary view

:organization: Logilab
:copyright: 2013 LOGILAB S.A. (Paris, FRANCE), license is LGPL.
:contact: https://www.logilab.fr/ -- mailto:contact@logilab.fr
"""

__docformat__ = "restructuredtext en"

from logilab.common.decorators import monkeypatch
from logilab.mtconverter import xml_escape

from cubicweb.web.views import primary


@monkeypatch(primary.PrimaryView)
def render_entity(self, entity):
    self.render_entity_toolbox(entity)
    self.render_entity_title(entity)
    # entity's attributes and relations, excluding meta data
    # if the entity isn't meta itself
    if self.is_primary():
        boxes = self._prepare_side_boxes(entity)
    else:
        boxes = None
    if boxes or hasattr(self, "render_side_related"):
        self.w('<div class="row">' '<div class="col-md-8">')
    self.content_navigation_components("navcontenttop")
    self.render_entity_attributes(entity)
    if self.main_related_section:
        self.render_entity_relations(entity)
    self.content_navigation_components("navcontentbottom")
    # side boxes
    if boxes or hasattr(self, "render_side_related"):
        self.w("</div>" '<div class="col-md-4">')  # </col-md-8>
        self.render_side_boxes(boxes)
        self.w("</div>" "</div>")  # </col-md-4>  # </row>


@monkeypatch(primary.PrimaryView)
def render_entity_attributes(self, entity):
    """Renders all attributes and relations in the 'attributes' section."""
    display_attributes = []
    for rschema, _, role, dispctrl in self._section_def(entity, "attributes"):
        vid = dispctrl.get("vid", "reledit")
        if rschema.final or vid == "reledit" or dispctrl.get("rtypevid"):
            value = entity.view(
                vid, rtype=rschema.type, role=role, initargs={"dispctrl": dispctrl}
            )
        else:
            rset = self._relation_rset(entity, rschema, role, dispctrl)
            if rset:
                value = self._cw.view(vid, rset)
            else:
                value = None
        if value is not None and value != "":
            display_attributes.append((rschema, role, dispctrl, value))
    if display_attributes:
        # bs customization begins
        self.w('<table class="table cw-table-primary-entity">')
        # bs customization ends
        for rschema, role, dispctrl, value in display_attributes:
            label = self._rel_label(entity, rschema, role, dispctrl)
            self.render_attribute(label, value, table=True)
        self.w("</table>")


@monkeypatch(primary.PrimaryView)
def render_relation(self, label, value):
    self.w('<div class="panel panel-default relations">')
    if label:
        self.w(
            '<div class="panel-heading"><div class="panel-title">%s</div></div>', label
        )
    self.w('<div class="panel-body">')
    self.w(value)
    self.w("</div>")
    self.w("</div>")


@monkeypatch(primary.RelatedView)
def call(self, **kwargs):
    if "dispctrl" in self.cw_extra_kwargs:
        if "limit" in self.cw_extra_kwargs["dispctrl"]:
            limit = self.cw_extra_kwargs["dispctrl"]["limit"]
        else:
            limit = self._cw.property_value("navigation.related-limit")
        list_limit = self.cw_extra_kwargs["dispctrl"].get("use_list_limit", 5)
        subvid = self.cw_extra_kwargs["dispctrl"].get("subvid", "incontext")
    else:
        limit = list_limit = None
        subvid = "incontext"
    if limit is None or self.cw_rset.rowcount <= limit:
        if self.cw_rset.rowcount == 1:
            self.wview(subvid, self.cw_rset, row=0)
        elif list_limit is None or 1 < self.cw_rset.rowcount <= list_limit:
            self.wview("csv", self.cw_rset, subvid=subvid)
        else:
            # bs customization begins
            self.wview("list", self.cw_rset, subvid=subvid, klass="list-unstyled")
            # bs customization ends
    # else show links to display related entities
    else:
        rql = self.cw_rset.printable_rql()
        rset = self.cw_rset.limit(limit)  # remove extra entity
        if list_limit is None:
            self.wview("csv", rset, subvid=subvid)
            self.w(
                '<a href="%s" class="see-all">%s</a>',
                self._cw.build_url(rql=rql, vid="list", subvid=subvid),
                self._cw._("see them all"),
            )
        else:
            self.w("<div>")
            # bs customization begins
            self.wview("list", rset, subvid=subvid, klass="list-unstyled")
            # bs customization ends
            self.w(
                '<a href="%s" class="see-all">%s</a>',
                self._cw.build_url(rql=rql, vid="list", subvid=subvid),
                self._cw._("see them all"),
            )
            self.w("</div>")



"""fluid_design_system implementation of htmlwidgets

:organization: Logilab
:copyright: 2013 LOGILAB S.A. (Paris, FRANCE), license is LGPL.
:contact: https://www.logilab.fr/ -- mailto:contact@logilab.fr
"""

__docformat__ = "restructuredtext en"

from logilab.common.decorators import monkeypatch

from cubicweb.web import htmlwidgets
from cubicweb.web.component import Separator


@monkeypatch(Separator)
def render(self, w):
    w('<li class="divider"></li>')


def bwcompatible_render_item(w, item):
    if hasattr(item, "render"):
        if getattr(item, "newstyle", False):
            if isinstance(item, Separator):
                item.render(w)
            else:
                w("<li>")
                item.render(w)
                w("</li>")
        else:
            item.render(w)  # XXX displays <li> by itself
    else:
        w("<li>%s</li>" % item)


@monkeypatch(htmlwidgets.BoxMenu)
def _render(self):
    tag = "li" if self.isitem else "div"
    self.w('<%s class="dropdown">', tag)
    self.w(
        '<a class="dropdown-toggle" data-toggle="dropdown" href="#">'
        "%s&nbsp;"
        '<span class="caret"></span>'
        "</a>",
        self.label,
        escape=False,
    )
    self.w('<ul class="dropdown-menu">')
    for item in self.items:
        bwcompatible_render_item(self.w, item)
    self.w("</ul>")
    self.w("</%s>", tag)
